Class: Primer::Alpha::IncludeFragment

Inherits:
Component
  • Object
show all
Defined in:
app/components/primer/alpha/include_fragment.rb

Overview

Use ‘IncludeFragment` to load HTML elements from the server. Add additional usage considerations or best practices that may aid the user to use the component correctly.

Constant Summary

Constants inherited from Component

Component::INVALID_ARIA_LABEL_TAGS

Constants included from Status::Dsl

Status::Dsl::STATUSES

Constants included from ViewHelper

ViewHelper::HELPERS

Constants included from TestSelectorHelper

TestSelectorHelper::TEST_SELECTOR_TAG

Constants included from FetchOrFallbackHelper

FetchOrFallbackHelper::InvalidValueError

Constants included from Primer::AttributesHelper

Primer::AttributesHelper::PLURAL_ARIA_ATTRIBUTES, Primer::AttributesHelper::PLURAL_DATA_ATTRIBUTES

Instance Method Summary collapse

Methods inherited from Component

deprecated?, generate_id

Methods included from JoinStyleArgumentsHelper

#join_style_arguments

Methods included from TestSelectorHelper

#add_test_selector

Methods included from FetchOrFallbackHelper

#fetch_or_fallback, #fetch_or_fallback_boolean, #silence_deprecations?

Methods included from ClassNameHelper

#class_names

Methods included from Primer::AttributesHelper

#aria, #data, #extract_data, #merge_aria, #merge_data, #merge_prefixed_attribute_hashes

Methods included from ExperimentalSlotHelpers

included

Methods included from ExperimentalRenderHelpers

included

Constructor Details

#initialize(src: nil, loading: :eager, **system_arguments) ⇒ IncludeFragment

Returns a new instance of IncludeFragment.

Parameters:

  • src (String) (defaults to: nil)

    The URL from which to retrieve an HTML element fragment.

  • loading (Symbol) (defaults to: :eager)

    <%= one_of([:lazy, :eager]) %>

  • system_arguments (Hash)

    <%= link_to_system_arguments_docs %>



14
15
16
17
18
19
# File 'app/components/primer/alpha/include_fragment.rb', line 14

def initialize(src: nil, loading: :eager, **system_arguments)
  @system_arguments = system_arguments
  @system_arguments[:tag] = "include-fragment"
  @system_arguments[:loading] = loading
  @system_arguments[:src] = src
end

Instance Method Details

#callObject



21
22
23
# File 'app/components/primer/alpha/include_fragment.rb', line 21

def call
  render(Primer::BaseComponent.new(**@system_arguments)) { content }
end