Class: Primer::OpenProject::DragHandle

Inherits:
Component
  • Object
show all
Defined in:
app/components/primer/open_project/drag_handle.rb

Overview

Add a general description of component here Add additional usage considerations or best practices that may aid the user to use the component correctly.

Constant Summary collapse

DEFAULT_SIZE =
Primer::Beta::Octicon::SIZE_DEFAULT
SIZE_OPTIONS =
Primer::Beta::Octicon::SIZE_OPTIONS

Constants inherited from Component

Component::INVALID_ARIA_LABEL_TAGS

Constants included from Status::Dsl

Status::Dsl::STATUSES

Constants included from ViewHelper

ViewHelper::HELPERS

Constants included from TestSelectorHelper

TestSelectorHelper::TEST_SELECTOR_TAG

Constants included from FetchOrFallbackHelper

FetchOrFallbackHelper::InvalidValueError

Constants included from AttributesHelper

AttributesHelper::PLURAL_ARIA_ATTRIBUTES, AttributesHelper::PLURAL_DATA_ATTRIBUTES

Instance Method Summary collapse

Methods inherited from Component

deprecated?, generate_id

Methods included from JoinStyleArgumentsHelper

#join_style_arguments

Methods included from TestSelectorHelper

#add_test_selector

Methods included from FetchOrFallbackHelper

#fetch_or_fallback, #fetch_or_fallback_boolean, #silence_deprecations?

Methods included from ClassNameHelper

#class_names

Methods included from AttributesHelper

#aria, #data, #extract_data, #merge_aria, #merge_data, #merge_prefixed_attribute_hashes

Methods included from ExperimentalSlotHelpers

included

Methods included from ExperimentalRenderHelpers

included

Constructor Details

#initialize(size: Primer::OpenProject::DragHandle::DEFAULT_SIZE, label: I18n.t("drag_handle.button_drag"), **system_arguments) ⇒ DragHandle

Returns a new instance of DragHandle.

Parameters:

  • size (Symbol) (defaults to: Primer::OpenProject::DragHandle::DEFAULT_SIZE)

    <%= one_of(Primer::OpenProject::DragHandle::SIZE_OPTIONS) %>

  • label (String) (defaults to: I18n.t("drag_handle.button_drag"))

    String that can be read by assistive technology. A label should be short and concise. See the accessibility section for more information.

  • system_arguments (Hash)

    <%= link_to_system_arguments_docs %>



17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
# File 'app/components/primer/open_project/drag_handle.rb', line 17

def initialize(size: Primer::OpenProject::DragHandle::DEFAULT_SIZE, label: I18n.t("drag_handle.button_drag"), **system_arguments)
  @system_arguments = deny_tag_argument(**system_arguments)

  deny_aria_key(
    :label,
    "instead of `aria-label`, include `label` on the component initializer.",
    **@system_arguments
  )
  @system_arguments[:tag] = "div"
  @system_arguments[:role] = "button"
  @system_arguments[:tabindex] = 0
  @system_arguments[:classes] = class_names(
    @system_arguments[:classes],
    "DragHandle"
  )

  @system_arguments[:aria] = merge_aria(
    @system_arguments,
    aria: { label: label }
  )

  @size = fetch_or_fallback(Primer::OpenProject::DragHandle::SIZE_OPTIONS, size, Primer::OpenProject::DragHandle::DEFAULT_SIZE)
end