Class: Primer::Alpha::IncludeFragment
- Defined in:
- app/components/primer/alpha/include_fragment.rb
Overview
Use IncludeFragment to load HTML elements from the server. Add additional usage considerations or best practices that may aid the user to use the component correctly.
Constant Summary
Constants inherited from Component
Component::INVALID_ARIA_LABEL_TAGS
Constants included from Status::Dsl
Constants included from ViewHelper
Constants included from TestSelectorHelper
TestSelectorHelper::TEST_SELECTOR_TAG
Constants included from FetchOrFallbackHelper
FetchOrFallbackHelper::InvalidValueError
Constants included from Primer::AttributesHelper
Primer::AttributesHelper::PLURAL_ARIA_ATTRIBUTES, Primer::AttributesHelper::PLURAL_DATA_ATTRIBUTES
Instance Method Summary collapse
- #call ⇒ Object
-
#initialize(src: nil, loading: :eager, **system_arguments) ⇒ IncludeFragment
constructor
A new instance of IncludeFragment.
Methods inherited from Component
Methods included from JoinStyleArgumentsHelper
Methods included from TestSelectorHelper
Methods included from FetchOrFallbackHelper
#fetch_or_fallback, #fetch_or_fallback_boolean, #silence_deprecations?
Methods included from ClassNameHelper
Methods included from Primer::AttributesHelper
#aria, #data, #extract_data, #merge_aria, #merge_data, #merge_prefixed_attribute_hashes
Methods included from ExperimentalSlotHelpers
Methods included from ExperimentalRenderHelpers
Constructor Details
#initialize(src: nil, loading: :eager, **system_arguments) ⇒ IncludeFragment
Returns a new instance of IncludeFragment.
14 15 16 17 18 19 |
# File 'app/components/primer/alpha/include_fragment.rb', line 14 def initialize(src: nil, loading: :eager, **system_arguments) @system_arguments = system_arguments @system_arguments[:tag] = "include-fragment" @system_arguments[:loading] = loading @system_arguments[:src] = src end |
Instance Method Details
#call ⇒ Object
21 22 23 |
# File 'app/components/primer/alpha/include_fragment.rb', line 21 def call render(Primer::BaseComponent.new(**@system_arguments)) { content } end |